# -*- coding: utf-8; tab-width: 4; indent-tabs-mode: nil -*-
#
# Copyright (C) 2009 - Jeremy AustinBardo <ausimage@ubuntu.com>;
#   Neil Shepperd <nshepperd@gmail.com>
# Copyright (C) 2006 - Florian Ludwig <dino@phidev.info>;
#   Franz Pletz <fpletz@franz-pletz.org>
#
# Apt-zeroconf is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Apt-zeroconf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with apt-zeroconf; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
Configuration Access Module for apt-zeroconf. Reads an ini style configuration
file with two sections: [apt-zeroconf] and [apt-cache].

@requires: ConfigParser
@requires: os.path.isfile
"""
import logging, os
from ConfigParser import SafeConfigParser

CONFIG_FILE = ("support/apt-zeroconf.conf"
                if os.path.isfile("support/apt-zeroconf.conf")
                else "/etc/apt-zeroconf.conf"
              )

logging.info('Using configuration file ' + CONFIG_FILE)

class Config(SafeConfigParser):
    """
    Basic avahi-zeroconf configuration values.

    @requires: ConfigParser.SafeConfigParser
    """
    __section = 'apt-zeroconf'
    def __init__(self, filename=CONFIG_FILE):
        """
        Set default values then load configuration from file.
        @param filename: filename
        @type filename: basestring

        @requires: socket.gethostname
        """
        from socket import gethostname
        SafeConfigParser.__init__(self, {
            'http_port': 1618,
            'apt-cache-path': '/var/cache/apt/archives',
            #'avahi_name': 'apt-zeroconf repository on %%s',
            'hostname': gethostname(),
            'stats': 'on',
            'restrict': 'off',
            })
        self.read(filename)

    def __getitem__(self, name):
        """
        Get value set in configuration.
        @param name: configuration key
        @type name: string
        @return: Configuration value for a given key
        @rtype: basestring
        """
        if self.has_option(self.__section, name):
            s = self.get(self.__section, name)
            return int(s) if s.isdigit() else s
        return None

    def set(self, name, value):
        """
        Set (that is, override) a value in configuration.
        @param name: configuration key
        @type name: string
        @param value: configuration value
        @type value: string
        """
        SafeConfigParser.set(self, self.__section, name, value)

class Cache(SafeConfigParser):
    """
    Proxy Urls used avahi-zeroconf in redirects and restricted proxies.

    @requires: ConfigParser.SafeConfigParser
    """
    __section = 'apt-cache'
    def __init__(self, filename=CONFIG_FILE):
        """
        Set default values then load configuration from file.
        @param filename: filename
        @type filename: basestring

        @requires: socket.gethostname
        """
        SafeConfigParser.__init__(self)
        self.read(filename)

    def __getitem__(self, name):
        """
        Get value set in configuration.
        @param name: configuration key
        @type name: string
        @return: Configuration value for a given key
        @rtype: basestring
        """
        if self.has_option(self.__section, name):
            s = self.get(self.__section, name)
            return [r.rstrip('/') for r in s.split()]
        return None

    def values(self):
        """
        List of values found in apt-cache section of configuration file.

        @return: apt-cache urls declared
        @rtype: list
        """
        return [self[key]
            for key, value in SafeConfigParser.items(self, self.__section)]

    def keys(self):
        """
        List of keys found in apt-cache section of configuration file.

        @return: apt-cache names declared
        @rtype: list
        """
        return [key
            for key, value in SafeConfigParser.items(self, self.__section)]

config = Config()
cache = Cache()
